# Change Log

All notable changes to this project will be documented in this file.

The format is based on [Keep a Changelog](http://keepachangelog.com/) 
and this project adheres to [Semantic Versioning](http://semver.org/).

## Development

## v2.13.0

### Added

- config field `customUderId` which can be used like the field `userId`
- send additional `closed` sample when page unloads

### Fixed

- Dash.js: state machine stuck when `PLAY` event comes before `READY` event (AN-1323)

### Changed

- Hls.js: only transition into error state if error is a fatal one (AN-1421)

## v2.12.0

### Added 

- Video.js: Track playback start failure and reason (AN-1326)
- Shaka & Dash.js: Track playback start failure and reason (AN-1292 & AN-1327)
- Hls.js: Track playback start failure and reason (AN-1291)

## v2.11.0

### Added

- Enabled incremental [3s, 5s, 10s, 30s, 60s] heartbeat while in rebuffer state

## v2.10.0

### Added
- audio- and subtitleLanguage tracking for hls.js (AN-1027)

### Fixed
- Added `onstatechange` event listener in `ShakaInternalAdapter`, as newer versions of `ShakaPlayer` return `null` for `getMediaElement` until the element is fully attached ([AN-1279](https://bitmovin.atlassian.net/browse/AN-1279))

## v2.9.0

### Added
- Bitmovin8: Track playback start failure and reason (AN-1082)

### Fixed
- window `close` event handler has been overwritten by the ad module
- Inconsistent order during the initialization of adapters and analytics module (AN-1128)

## v2.8.1

### Fixed
- Hlsjs: error events were not registered by analytics HlsAdapter
- Hlsjs: isLive check throws unhandled error and cancled sending of sample 

## v2.8.0

### Added
- `customData6` and `customData7` can be passed with the analytics config (AN-1007)
- Bandwidth measurements for hls.js

### Changed
- log level to `warning` of failed license calls due to `Do Not Track` enabled (AN-985)
- internal typings for Bitmovin7 and HTML5 Statemachine

## v2.7.1

### Fixed
- Sharing cookies across the same domain was not working (AN-955)
- Set cookie expiration date to 10 years (AN-955)

## v2.7.0

### Added
- VideoJS: Track selected subtitle with `subtitleEnabled` and `subtitleLanguage` (AN-917).
- VideoJS: Track selected audioLanguage with `audioLanguage` in the sample (AN-918).

### Fixed
- VideoJS: Correctly track seek time

## v2.6.0

### Added
- `isLive=true|false` to Analytics config to indicate if the video is live. Will be overriden once playback data is available.

### Removed
- Unused config variables `ads` and `cast`

## v2.5.4

### Fixed
- TypeScript typings were missing default export

## v2.5.3

### Fixed
- Typings can now be imported without depending on all supported players' types

### Added
- TypeScript typings (`.d.ts` files)
- Source map files to CDN release

## v2.5.2

### Fixed

- Crash on IE11 due to `String.endsWith` not being available.
- Removed outdated `tests` and unused `test-suite`.
- When changing sources, the collector `config` is lost if none is passed
- `Origin` from `collectorConfig` is not updated after sourceChange

### Known Issues

- VideoJS cannot correctly detect autoplay (AN-470210111)
- VideoJS doesn't support correct startup time tracking for source changes (AN-470210083)
- VideoJS: Metadata from the first source change is ignored (AN-470210066)

## v2.5.1

### Changed

- VideoJS rebuffering measurement method to be more accurate

### Added

- state `initial_source_change` to send correct analytics data in samples when the config changes from `player.src` to `player.load`
- `sourceChange` event handling for VideoJS

### Fixed

- `droppedFrames` is always `0`.
- For VideoJS player: Prevent Exception when the HLS `RESOLUTION` is undefined

## v2.5.0

### Added

- `origin` to Analytics Collector configuration to set the origin on samples for devices that do not send the origin header in requests.
- `cookiesEnabled=true|false` to Analytics Collector configuration to disable cookies. Defaults to `true`. If cookies are disabled, `userId` is not persisted across collector sessions.
- For Bitmovin Player v7 and v8: The selected audio language is available as `audioLanguage` in the sample.
- For Bitmovin Player v7 and v8: Sample now contains `subtitleEnabled` and `subtitleLanguage`.

### Fixed

- `timeInViewport` wasn't calculated correctly if ad playback was initially started outside of the viewport.

## v2.4.3

### Added

- Exception handling in case a HTTP call fails.

## v2.4.2

### Fixed

- If using deferred license loading, the license key wasn't updated in the samples.

## v2.4.1

### Added

- For Bitmovin Player v8, if the player config contains `analytics: false`, the Analytics module will never be initialized.

### Fixed

- `window.setInterval` leak when player.destroy was called in the `adstarted` callback.
- Removed `console.error` on failed license response from player.

## v2.4.0

### Added

- TypeScript declaration files

### Fixed 

- Collector Sessions that started with `enabled: false` did not report `playerStartupTime`, `pageLoadTime`, `startupTime`. They now get sent with the first enabled startup sample.
- Starting with Bitmovin Player v8.10, Analytics will get the analytics license key automatically from the player, if a default license is specified in the dashboard.

## v2.3.0

### Added

- `audioCodec` and `videoCodec` to outgoing payload
- `supportedVideoCodecs` to outgoing payload

### Fixed

- In Safari, the `IntersectionObserver` class doesn't exist and that caused a crash in the Ad module

## v2.2.0 - 25/01/2019

### Added

- Added `config` to analytics configuration
- Configure backend url via `analyticsConfig.config.backendUrl`
- Error segment tracking.
- Sample now contains `errorData` JSON with additional information for Bitmovin 8 errors.
- Player v8.3 compatibility
- Improvements to the Ad Module with 8.3 (FallbackIndex and Waterfall level)

## v2.1.1 - 17/01/2019

### Added

- `version` property on all analytics instances to get analytics collector version
- `player.analytics.version` property to get analytics collector version
- `window.bitmovin.analytics.version` property to get analytics collector version

### Fixed

- Bitmovin collector no longer pollutes the global scope when used in a requirejs/es6 context

## v2.1.0 - 14/01/2019

### Added

- Added ads tracking (Bitmovin 8 only)
- Added bandwith measurements
- Sample now sends `web` in a platform field 
- AdSample now sends `web` in in the platform field 
- Sample now contains a `sequenceNumber` field

### Changes

- Bitmovin7Adapter now checks if another analytics collector is already registered up

### Fixed

- Looping a video (Calling `.play()` after `playbackFinished` occured) did not correctly generate a new impressionId

## v2.0.0 - 20/11/2018

### Changed

- Setup API has been reworked to use JavaScript classes and specific adapters per player technology

### Fixed

- Video Bitrate is now correctly reported before the first quality change events in all adapters
- Player version reporting has been improved.
- Fixed incorrect handling of `videoWindowWidth` and `videoPlaybackWidth` in HLS.js and Shaka.
- When loading a new video with Bitmovin Player v7 and v8, a new impression id was assigned before the last sample of the old impression has been sent to the server.
- Fixed error being thrown when `hls.destroy()` is called because of a not correctly cleared timeout.

### Added

- `AnalyticsPlayerModule` class
- `Analytics.version` returns the version of the analytics collector
- Added support for native HTML5 Video playback via the `HTMLVideoElementAdapter`
- Added specific adapter classes for each supported player technology
  - `Bitmovin8Adapter`
  - `Bitmovin7Adapter`
  - `VideojsAdapter`
  - `ShakaAdapter`
  - `HlsAdapter`
  - `DashjsAdapter`
  - `HTMLVideoElementAdapter`

### Removed

- `.augment` syntax removed in favor of ES6 classes
- `.register` syntax removed in favor of ES6 classes 
- `window.bitmovin.analytics`
- `window.bitmovin.analytics.Players`
- `window.bitmovin.analytics.CdnProviders`
- `window.bitmovin.analytics.PlayerModule`
- `window.bitmovin.analytics.version`
- Automatic Player detection is deprecated in favor of explicit Adapter creation

### Known Issues

- `getPlayerVersion` will return `unkown` for video.js if included via npm or requirejs
- Not calling `player.load` in the same code-path after `new bitmovin.player.Player()` will skew the `PLAYER_STARTUPTIME` metric.
- Bitrate Information is not collected when using Native Safari Playback using the `HTMLVideoElementAdapter`. This is due to the limitations of the Browser and not a bug.
- The HTML5 adapters and the VideoJs adapter won't assign a new impression id on `Analytics.sourceChange()`
- Destroying a Shaka instance will not correctly clear all event handlers on the VideoElement. Re-Using the VideoElement with another Shaka instance will lead to the impression being tracked twice

## [v1.7.4] - 24/10/2018

### Fixed

- Player detection for Shaka when loaded via require and `window.Shaka` is not available.
- Player detection for HLS.js when loaded via require and `window.HLS` is not available.
- Beta-Support for Shaka 2.3.7
- Beta-Support for HLS.js 0.7.11

### Known Issues

- Shaka/HLS Adapter: Video width/heigth/bitrate is only available after the first videoQuality change event.

## [v1.7.3] - 5/10/2018

### Fixed
- `pageLoadType` was not correctly detected for subsequent sessions in the same player
- `playerStartupTime` was reported as `NULL` instead of `0` for subsequent sessions in the same player
- Internal improvements and refactorings

## [v1.7.2] - 21/09/2018

### Fixed
- `isLive` detection was broken in subsequent sessions on the same player

## [v1.7.1] - 10/09/2018

### Added
- added default export for es6 style imports

## [v1.7.0] - 07/09/2018

### Added
- Bitmovin Player v8 Support
- `title` configuration attribute
- introduced new [new "embedded" configuration API](https://bitmovin.com/docs/analytics/faqs/collector-v1-7-embedded-analytics-configuration-for-bitmovin-player)
- DRM performance tracking (`drmLoadType`, `drmLoadTime`)

### Removed

- Bitmovin Player v6 support
- Chromecast receiver side support

## [v1.6.0] - 23/04/2018

### Added
- DASH.js support
- HLS.js support
- Shaka player support
- HTML5 media element support

### Changed
- added source maps to the NPM package

### Fixed
- inaccuracies in seek time tracking
- video.js adapter bugfixes
